// PIC16F628A Tachometer, HT1611
// CCS C Compiler, PCWHD v5.101
// 7.07.2023, LZ2WSG, KN34PC
//---------------------------------------------------------------------------------------------------------
#include <main.h>
#include <stdint.h>
#include <stdbool.h>

// PIC16F628A
//                _____
//       PIN_A2 -|  ^  |- PIN_A1
//       PIN_A3 -|     |- PIN_A0
// T0CKI/PIN_A4 -|     |- PIN_A7/OSC1/CLKIN
//  MCLR/PIN_A5 -|     |- PIN_A6/OSC2/CLKOUT
//          VSS -|     |- VDD
//   INT/PIN_B0 -|     |- PIN_B7/PGD
//       PIN_B1 -|     |- PIN_B6/PGC
//       PIN_B2 -|     |- PIN_B5
//       PIN_B3 -|_____|- PIN_B4

uint16_t t1 = 0;     // timer1 overflows
uint16_t t1_hi = 0;  // timer1 high
uint16_t t1_lo = 0;  // timer1 low

void write_digit(int8_t d);
void show_display(uint16_t d);
//---------------------------------------------------------------------------------------------------------
void main() {
  enable_interrupts(INT_EXT_L2H);

  setup_timer_0(T0_INTERNAL | T0_DIV_256);
  enable_interrupts(INT_TIMER0);

  setup_timer_1(T1_INTERNAL | T1_DIV_BY_1);
  enable_interrupts(INT_TIMER1);

  enable_interrupts(GLOBAL);

  output_low(CLK);
  delay_ms(1000);

  while (TRUE) {
  }
}
//---------------------------------------------------------------------------------------------------------
// 4000000 / 4 / 256 / 256 = 15.2587890625 Hz
#INT_TIMER0
void TMR0_ISR(void) {
  static uint8_t t0 = 0;  // timer0 overflows
  static uint32_t tmp_old = 0;
  uint32_t tmp = 0;

  t0++;

  if (t0 > 13) {
    uint32_t t = make32(t1_hi, t1_lo);

    if (t > 0)
      tmp = (1000000 * 60) / t;
    else
      tmp = 0;

    if (tmp != tmp_old) {
      show_display((uint16_t)tmp);
      tmp_old = tmp;
    }

    t0 = 0;

    t1_hi = 0;
    t1_lo = 0;
  }
}
//---------------------------------------------------------------------------------------------------------
// 4000000 / 4 = 1000000 Hz, div 65536
#INT_TIMER1
void TMR1_ISR(void) {
  t1++;
}
//---------------------------------------------------------------------------------------------------------
#INT_EXT
void ext_isr(void) {
  if (input(TIN)) {
    t1_lo = get_timer1();
    set_timer1(0);

    t1_hi = t1;

    t1 = 0;
  }
}
//---------------------------------------------------------------------------------------------------------
void write_digit(int8_t d) {
  switch (d) {
    case  0: d = 0b00001010; break;  // 0
    case -1: d = 0b00000000; break;  // blank
  }

  for (uint8_t i = 0; i < 4; i++) {
    output_high(CLK);
    output_bit(DI, (d & 0x8) != 0);
    d <<= 1;
    delay_us(1);
    output_low(CLK);
    delay_us(5);
  }
}
//---------------------------------------------------------------------------------------------------------
void show_display(uint16_t d) {
  write_digit(-1);  // blank
  write_digit(-1);  // blank
  write_digit(-1);  // blank
  write_digit(d / 10000);
  write_digit((d / 1000) % 10);
  write_digit((d / 100) % 10);
  write_digit((d / 10) % 10);
  write_digit(d % 10);
  write_digit(-1);  // blank
  write_digit(-1);  // blank
}
